""" Known instances of eltern-portal.org """

# pylint: disable=too-many-lines

from .school import School

SCHOOLS = [
    School(
        school="abendrsau",
        name="Abendrealschule Augsburg",
        postcode="86152",
        city="Augsburg",
    ),
    School(
        school="abstgym",
        name="Anton-Bruckner-Gymnasium Straubing",
        postcode="94315",
        city="Straubing",
    ),
    School(
        school="aegymuc",
        name="Albert-Einstein-Gymnasium München",
        postcode="81545",
        city="München",
    ),
    School(
        school="agbersau",
        name="Agnes-Bernauer-Schule - Realschule für Mädchen",
        postcode="86152",
        city="Augsburg",
    ),
    School(
        school="akgkitz",
        name="Armin-Knab-Gymnasium Kitzingen",
        postcode="97318",
        city="Kitzingen",
    ),
    School(
        school="alblaugy",
        name="Albertus-Gymnasium Lauingen",
        postcode="89415",
        city="Lauingen (Donau)",
    ),
    School(
        school="alkemgym",
        name="Allgäu-Gymnasium Kempten",
        postcode="87435",
        city="Kempten (Allgäu)",
    ),
    School(
        school="amgydie",
        name="Ammersee-Gymnasium Dießen",
        postcode="86911",
        city="Dießen am Ammersee",
    ),
    School(
        school="amorgym",
        name="Karl-Ernst-Gymnasium Amorbach",
        postcode="63916",
        city="Amorbach",
    ),
    School(
        school="anfragym",
        name="Anne-Frank-Gymnasium Erding",
        postcode="85435",
        city="Erding",
    ),
    School(
        school="ansfobo",
        name="Berufliche Oberschule Ansbach",
        postcode="91522",
        city="Ansbach",
    ),
    School(
        school="apgying",
        name="Apian-Gymnasium Ingolstadt",
        postcode="85051",
        city="Ingolstadt",
    ),
    School(
        school="asamgymuc",
        name="Asam-Gymnasium München",
        postcode="81541",
        city="München",
    ),
    School(
        school="astipagym",
        name="Adalbert-Stifter-Gymnasium Passau",
        postcode="94032",
        city="Passau",
    ),
    School(
        school="aufreugy",
        name="Auersperg-Gymnasium",
        postcode="94034",
        city="Passau",
    ),
    School(
        school="augystan",
        name="Gymnasium bei St. Anna",
        postcode="86159",
        city="Augsburg",
    ),
    School(
        school="augysteph",
        name="Gymnasium bei St. Stephan Augsburg",
        postcode="86152",
        city="Augsburg",
    ),
    School(
        school="austafach",
        name="Staatsinstitut für die Ausbildung von Fachlehrern",
    ),
    School(
        school="avegybu",
        name="Aventinus-Gymnasium Burghausen",
        postcode="84489",
        city="Burghausen",
    ),
    School(
        school="avhgym",
        name="Alexander-von-Humboldt-Gymnasium Schweinfurt",
        postcode="97424",
        city="Schweinfurt",
    ),
    School(
        school="bagyeich",
        name="Eichendorff-Gymnasium Bamberg",
        postcode="96050",
        city="Bamberg",
    ),
    School(
        school="baiblgy",
        name="Gymnasium Bad Aibling",
        postcode="83043",
        city="Bad Aibling",
    ),
    School(
        school="baykola",
        name="Bayernkolleg Augsburg",
        postcode="86169",
        city="Augsburg",
    ),
    School(
        school="bbgymuc",
        name="Städtisches Bertolt-Brecht-Gymnasium München",
        postcode="81243",
        city="München",
    ),
    School(
        school="bngmh",
        name="Balthasar-Neumann-Gymnasium Marktheidenfeld",
        postcode="97828",
        city="Marktheidenfeld",
    ),
    School(
        school="bskoegy",
        name="Benedikt-Stattler-Gymnasium Bad Kötzting",
        postcode="93444",
        city="Bad Kötzting",
    ),
    School(
        school="bszhof2ws",
        name="Staatliche Wirtschaftsschule Hof",
        postcode="95028",
        city="Hof",
    ),
    School(
        school="bugymal",
        name="Burkhart-Gymnasium Mallersdorf-Pfaffenberg",
        postcode="84066",
        city="Mallersdorf-Pfaffenberg",
    ),
    School(
        school="bukugym",
        name="Gymnasium Burgkunstadt",
        postcode="96224",
        city="Burgkunstadt",
    ),
    School(
        school="casigym",
        name="Gymnasium Casimirianum Coburg",
        postcode="96450",
        city="Coburg",
    ),
    School(
        school="cavigy",
        name="Caspar-Vischer-Gymnasium Kulmbach",
        postcode="95326",
        city="Kulmbach",
    ),
    School(
        school="cjtgylau",
        name="Christoph-Jacob-Treu-Gymnasium Lauf",
        postcode="91207",
        city="Lauf an der Pegnitz",
    ),
    School(
        school="comgydeg",
        name="Comenius-Gymnasium Deggendorf",
    ),
    School(
        school="cpgygil",
        name="Christoph Probst Gymnasium Gilching",
        postcode="82205",
        city="Gilching",
    ),
    School(
        school="csgying",
        name="Christoph-Scheiner-Gymnasium",
        postcode="85049",
        city="Ingolstadt",
    ),
    School(
        school="dantemuc",
        name="Dante-Gymnasium München",
        postcode="81371",
        city="München",
    ),
    School(
        school="degfobo",
        name="Aloys-Fischer-Schule Deggendorf",
        postcode="94469",
        city="Deggendorf",
    ),
    School(
        school="degyaich",
        name="Deutschherren-Gymnasium Aichach",
        postcode="86551",
        city="Aichach",
    ),
    School(
        school="deigs",
        name="Grundschule Deisenhofen",
        postcode="82041",
        city="Oberhaching",
    ),
    School(
        school="deroko",
        name="Robert-Koch-Gymnasium Deggendorf",
        postcode="94469",
        city="Deggendorf",
    ),
    School(
        school="dilgyjms",
        name="Johann-Michael-Sailer-Gymnasium",
        postcode="89407",
        city="Dillingen a.d.Donau",
    ),
    School(
        school="dingogym",
        name="Gymnasium Dingolfing",
        postcode="84130",
        city="Dingolfing",
    ),
    School(
        school="domfreigy",
        name="Dom-Gymnasium Freising",
        postcode="85354",
        city="Freising",
    ),
    School(
        school="domgyreg",
        name="Gymnasium der Regensburger Domspatzen",
        postcode="93055",
        city="Regensburg",
    ),
    School(
        school="donaugym",
        name="Gymnasium Donauwörth",
        postcode="86609",
        city="Donauwörth",
    ),
    School(
        school="dorfgym",
        name="Gymnasium Dorfen",
        postcode="84405",
        city="Dorfen",
    ),
    School(
        school="dzglan",
        name="Dominikus-Zimmermann-Gymnasium Landsberg am Lech",
        postcode="86899",
        city="Landsberg am Lech",
    ),
    School(
        school="ebgymuc",
        name="Städtisches Elsa-Brändström-Gymnasium München",
        postcode="81241",
        city="München",
    ),
    School(
        school="eckegym",
        name="Gymnasium Eckental",
        postcode="90542",
        city="Eckental",
    ),
    School(
        school="ehrforgy",
        name="Ehrenbürg-Gymnasium Forchheim",
        postcode="91301",
        city="Forchheim",
    ),
    School(
        school="emagyha",
        name="Ernst-Mach-Gymnasium Haar",
        postcode="85540",
        city="Haar",
    ),
    School(
        school="engyerl",
        name="Emmy-Noether-Gymnasium Erlangen",
        postcode="91058",
        city="Erlangen",
    ),
    School(
        school="ergogym",
        name="Gymnasium Ergolding",
        postcode="84030",
        city="Ergolding",
    ),
    School(
        school="ergosfz",
        name="Sonderpädagogisches Förderzentrum Landshut-Land",
        postcode="84030",
        city="Ergolding",
    ),
    School(
        school="esgymuc",
        name="Erzbischöfliches Edith-Stein-Gymnasium",
        postcode="81667",
        city="München",
    ),
    School(
        school="etahgybam",
        name="E.T.A. Hoffmann-Gymnasium Bamberg",
        postcode="96049",
        city="Bamberg",
    ),
    School(
        school="evbspar",
        name="Emil-von-Behring-Gymnasium Spardorf",
    ),
    School(
        school="fagyneu",
        name="Friedrich-Alexander-Gymnasium",
        postcode="91413",
        city="Neustadt an der Aisch",
    ),
    School(
        school="ffbgrg",
        name="Graf-Rasso-Gymnasium",
        postcode="82256",
        city="Fürstenfeldbruck",
    ),
    School(
        school="finrogym",
        name="Sebastian-Finsterwalder-Gymnasium Rosenheim",
        postcode="83022",
        city="Rosenheim",
    ),
    School(
        school="fkgwue",
        name="Friedrich-Koenig-Gymnasium",
        postcode="97082",
        city="Würzburg",
    ),
    School(
        school="flgybam",
        name="Franz-Ludwig-Gymnasium Bamberg",
    ),
    School(
        school="framagym",
        name="Franz-Marc-Gymnasium",
        postcode="85570",
        city="Markt Schwaben",
    ),
    School(
        school="freicamgy",
        name="Camerloher-Gymnasium Freising",
        postcode="85354",
        city="Freising",
    ),
    School(
        school="freigymuc",
        name="Gymnasium Freiham",
        postcode="81248",
        city="München",
    ),
    School(
        school="fridgyerl",
        name="Gymnasium Fridericianum",
        postcode="19053",
        city="Schwerin",
    ),
    School(
        school="friebos",
        name="Berufliche Oberschule Friedberg",
        postcode="86316",
        city="Friedberg",
    ),
    School(
        school="friedgym",
        name="Gymnasium Friedberg",
        postcode="86316",
        city="Friedberg",
    ),
    School(
        school="fuerimuc",
        name="Gymnasium Fürstenried",
        postcode="81475",
        city="München",
    ),
    School(
        school="gagymain",
        name="Gabelsberger-Gymnasium Mainburg",
        postcode="84048",
        city="Mainburg",
    ),
    School(
        school="gargym",
        name="Gymnasium Gars",
        postcode="83536",
    ),
    School(
        school="gesaholl",
        name="Staatliche Gesamtschule Hollfeld",
    ),
    School(
        school="gisgymuc",
        name="Gisela-Gymnasium München",
        postcode="80801",
        city="München",
    ),
    School(
        school="gk",
        name="Gymnasium Kirchheim",
        postcode="85551",
        city="Kirchheim bei München",
    ),
    School(
        school="gmgbay",
        name="Graf-Münster-Gymnasium Bayreuth",
        postcode="95444",
        city="Bayreuth",
    ),
    School(
        school="gmshaag",
        name="Grund- und Mittelschule Haag i. OB",
        postcode="83527",
        city="Haag in Oberbayern",
    ),
    School(
        school="gmslaufen",
        name="Ruperti Grund- und Mittelschule Laufen",
        postcode="83410",
        city="Laufen",
    ),
    School(
        school="gn",
        name="Gymnasium Neubiberg",
        postcode="85579",
        city="Neubiberg",
    ),
    School(
        school="gnadgym",
        name="Gnadenthal-Gymnasium Ingolstadt der Diözese Eichstätt",
    ),
    School(
        school="go",
        name="Gymnasium Ottobrunn",
        postcode="85521",
        city="Ottobrunn",
    ),
    School(
        school="grafgym",
        name="Max-Mannheimer-Gymnasium Grafing",
        postcode="85567",
        city="Grafing bei München",
    ),
    School(
        school="groebegy",
        name="Gymnasium Gröbenzell",
        postcode="82194",
    ),
    School(
        school="gruewagym",
        name="Gymnasium Grünwald",
        postcode="82031",
        city="Grünwald",
    ),
    School(
        school="gsgstu",
        name="Geschwister-Scholl-Gymnasium Stuttgart-Sillenbuch",
        postcode="70619",
        city="Stuttgart",
    ),
    School(
        school="gsuhg",
        name="Grundschule am Sportpark",
    ),
    School(
        school="gykisee",
        name="Gymnasium Kirchseeon",
        postcode="85614",
    ),
    School(
        school="gymmod",
        name="Gymnasium Marktoberdorf",
        postcode="87616",
        city="Marktoberdorf",
    ),
    School(
        school="gympan",
        name="Gymnasium Pfarrkirchen",
        postcode="84347",
        city="Pfarrkirchen",
    ),
    School(
        school="gymzwie",
        name="Gymnasium Zwiesel",
        postcode="94227",
        city="Zwiesel",
    ),
    School(
        school="gypenz",
        name="Gymnasium Penzberg",
        postcode="82377",
        city="Penzberg",
    ),
    School(
        school="halegyla",
        name="Hans-Leinberger-Gymnasium Landshut",
        postcode="84034",
        city="Landshut",
    ),
    School(
        school="hamfrogy",
        name="Frobenius-Gymnasium Hammelburg",
        postcode="97762",
        city="Hammelburg",
    ),
    School(
        school="hcgla",
        name="Hans-Carossa-Gymnasium",
        postcode="14089",
        city="Berlin",
    ),
    School(
        school="hehegymuc",
        name="Städt. Heinrich-Heine-Gymnasium",
    ),
    School(
        school="helagyfue",
        name="Helene-Lange-Gymnasium Fürth",
        postcode="90762",
        city="Fürth",
    ),
    School(
        school="heraugy",
        name="Gymnasium Herzogenaurach",
        postcode="91074",
        city="Herzogenaurach",
    ),
    School(
        school="hertrogym",
        name="Hertzhaimer-Gymnasium Trostberg",
        postcode="83308",
        city="Trostberg",
    ),
    School(
        school="hilgykem",
        name="Hildegardis-Gymnasium Kempten",
        postcode="87439",
        city="Kempten (Allgäu)",
    ),
    School(
        school="hoesiegym",
        name="Gymnasium Höhenkirchen-Siegertsbrunn",
        postcode="85635",
        city="Höhenkirchen-Siegertsbrunn",
    ),
    School(
        school="holgyau",
        name="Holbein-Gymnasium",
        postcode="86150",
        city="Augsburg",
    ),
    School(
        school="holkigym",
        name="Gymnasium Holzkirchen",
        postcode="83607",
        city="Holzkirchen",
    ),
    School(
        school="hoschgym",
        name="Gymnasium Hohenschwangau",
        postcode="87645",
    ),
    School(
        school="hsgyerl",
        name="Hermann-Staudinger-Gymnasium Erlenbach a. Main",
        postcode="63906",
        city="Erlenbach am Main",
    ),
    School(
        school="hsgyfue",
        name="Heinrich-Schliemann-Gymnasium",
        postcode="90762",
        city="Fürth",
    ),
    School(
        school="hsgyhoe",
        name="Hanns-Seidel-Gymnasium Hösbach",
        postcode="63768",
        city="Hösbach",
    ),
    School(
        school="hugyvat",
        name="Humboldt-Gymnasium-Vaterstetten",
        postcode="85598",
        city="Vaterstetten",
    ),
    School(
        school="hvbrsau",
        name="Heinrich-von-Buz Realschule Augsburg",
        postcode="86154",
        city="Augsburg",
    ),
    School(
        school="ickigym",
        name="Rainer-Maria-Rilke-Gymnasium Icking",
        postcode="82057",
    ),
    School(
        school="igtadah",
        name="Ignaz-Taschner-Gymnasium",
        postcode="85221",
        city="Dachau",
    ),
    School(
        school="iguerogym",
        name="Ignaz-Günther-Gymnasium Rosenheim",
        postcode="83022",
        city="Rosenheim",
    ),
    School(
        school="illvoegy",
        name="Illertal-Gymnasium Vöhringen",
        postcode="89269",
        city="Vöhringen",
    ),
    School(
        school="indomagy",
        name="Gymnasium Markt Indersdorf",
        postcode="85229",
        city="Markt Indersdorf",
    ),
    School(
        school="irmrs",
        name="Erzbischöfliche St.-Irmengard-Realschule",
    ),
    School(
        school="jafugyau",
        name="Jakob-Fugger-Gymnasium Augsburg",
        postcode="86152",
        city="Augsburg",
    ),
    School(
        school="jcrgyhof",
        name="Johann-Christian-Reinhart-Gymnasium Hof",
        postcode="95030",
        city="Hof",
    ),
    School(
        school="joguwa",
        name="Johannes-Gutenberg-Gymnasium Waldkirchen",
        postcode="94065",
        city="Waldkirchen",
    ),
    School(
        school="joheitrau",
        name="Johannes-Heidenhain-Gymnasium Traunreut",
        postcode="83301",
        city="Traunreut",
    ),
    School(
        school="johofrei",
        name="Josef-Hofmiller-Gymnasium Freising",
        postcode="85354",
        city="Freising",
    ),
    School(
        school="jonegyro",
        name="Johannes-Nepomuk-Gymnasium",
        postcode="93352",
    ),
    School(
        school="jpgyho",
        name="Jean-Paul-Gymnasium Hof",
        postcode="95028",
        city="Hof",
    ),
    School(
        school="jsgbk",
        name="Jack-Steinberger-Gymnasium Bad Kissingen",
        postcode="97688",
        city="Bad Kissingen",
    ),
    School(
        school="jsgkarl",
        name="Johann-Schöner-Gymnasium Karlstadt",
        postcode="97753",
        city="Karlstadt",
    ),
    School(
        school="julineu",
        name="Justus-von-Liebig-Gymnasium",
        postcode="86356",
        city="Neusäß",
    ),
    School(
        school="jvfgcham",
        name="Joseph-von-Fraunhofer-Gymnasium Cham",
        postcode="93413",
        city="Cham",
    ),
    School(
        school="kagerd",
        name="Korbinian-Aigner-Gymnasium Erding",
        postcode="85435",
        city="Erding",
    ),
    School(
        school="karlsgymuc",
        name="Karlsgymnasium München",
        postcode="81243",
        city="München",
    ),
    School(
        school="karogyro",
        name="Karolinen-Gymnasium Rosenheim",
        postcode="83022",
        city="Rosenheim",
    ),
    School(
        school="kepgywei",
        name="Kepler-Gymnasium Weiden",
        postcode="92637",
        city="Weiden in der Oberpfalz",
    ),
    School(
        school="kgbarei",
        name="Karlsgymnasium Bad Reichenhall",
        postcode="83435",
        city="Bad Reichenhall",
    ),
    School(
        school="khgygrae",
        name="Kurt-Huber-Gymnasium Gräfelfing",
        postcode="82166",
        city="Gräfelfing",
    ),
    School(
        school="klengymuc",
        name="Klenze-Gymnasium München",
        postcode="81371",
        city="München",
    ),
    School(
        school="koebrugy",
        name="Gymnasium Königsbrunn",
        postcode="86343",
        city="Königsbrunn",
    ),
    School(
        school="krogya",
        name="Kronberg-Gymnasium Aschaffenburg",
        postcode="63739",
        city="Aschaffenburg",
    ),
    School(
        school="ksigym",
        name="Kolleg der Schulbrüder Illertissen",
        postcode="89257",
        city="Illertissen",
    ),
    School(
        school="kumagybu",
        name="Kurfürst-Maximilian-Gymnasium Burghausen",
        postcode="84489",
        city="Burghausen",
    ),
    School(
        school="kvc",
        name="Karl-von-Closen-Gymnasium Eggenfelden",
    ),
    School(
        school="lalegygra",
        name="Landgraf-Leuchtenberg-Gymnasium Grafenau",
        postcode="94481",
        city="Grafenau",
    ),
    School(
        school="landau",
        name="Gymnasium Landau a.d. Isar",
        postcode="94405",
        city="Landau an der Isar",
    ),
    School(
        school="leigyalt",
        name="Leibniz-Gymnasium Altdorf",
        postcode="90518",
        city="Altdorf bei Nürnberg",
    ),
    School(
        school="lessgynu",
        name="Lessing-Gymnasium Neu-Ulm",
        postcode="89231",
        city="Neu-Ulm",
    ),
    School(
        school="liefraurs",
        name="Liebfrauenschule Diessen",
        postcode="86911",
        city="Dießen am Ammersee",
    ),
    School(
        school="lifegymuc",
        name="Städtisches Lion-Feuchtwanger-Gymnasium München",
        postcode="80807",
        city="München",
    ),
    School(
        school="lindgym",
        name="Gymnasium Lindenberg",
        postcode="88161",
        city="Lindenberg im Allgäu",
    ),
    School(
        school="lmguhag",
        name="Lise-Meitner-Gymnasium Unterhaching",
        postcode="82008",
        city="Unterhaching",
    ),
    School(
        school="losgymuc",
        name="Städtisches Louise-Schroeder-Gymnasium München",
        postcode="80999",
        city="München",
    ),
    School(
        school="lshmar",
        name="Gymnasium Landschulheim Marquartstein",
        postcode="83250",
    ),
    School(
        school="lsising",
        name="Landschulheim Schloss Ising am Chiemsee",
        postcode="83339",
    ),
    School(
        school="lskempf",
        name="Landschulheim Kempfenhausen",
        postcode="82335",
    ),
    School(
        school="ltgyprie",
        name="Ludwig-Thoma-Gymnasium",
        postcode="83209",
        city="Prien am Chiemsee",
    ),
    School(
        school="ludgymuc",
        name="Ludwigsgymnasium München",
        postcode="81377",
        city="München",
    ),
    School(
        school="ludgystra",
        name="Ludwigsgymnasium Straubing",
        postcode="94315",
        city="Straubing",
    ),
    School(
        school="luigywas",
        name="Luitpold-Gymnasium Wasserburg am Inn",
        postcode="83512",
        city="Wasserburg am Inn",
    ),
    School(
        school="luise",
        name="Städtisches Luisengymnasium",
        postcode="80333",
        city="München",
    ),
    School(
        school="lupomuc",
        name="Luitpold-Gymnasium München",
        postcode="80538",
        city="München",
    ),
    School(
        school="lwgschwab",
        name="Leonhard-Wagner-Gymnasium Schwabmünchen",
    ),
    School(
        school="mabogym",
        name="Max-Born-Gymnasium",
        postcode="82110",
        city="Germering",
    ),
    School(
        school="marbugym",
        name="Marianum Buxheim Gymnasium des Schulwerks der Diözese Augsburg",
        postcode="87740",
        city="Buxheim",
    ),
    School(
        school="marktgym",
        name="Gymnasium Marktbreit",
        postcode="97340",
        city="Marktbreit",
    ),
    School(
        school="mathemuc",
        name="Maria-Theresia-Gymnasium",
        postcode="81541",
        city="München",
    ),
    School(
        school="maxgymuc",
        name="Maximiliansgymnasium München",
        postcode="80803",
        city="München",
    ),
    School(
        school="mgf",
        name="Maristengymnasium Fürstenzell",
        postcode="94081",
    ),
    School(
        school="mggwue",
        name="Matthias-Grünewald-Gymnasium Würzburg",
        postcode="97074",
        city="Würzburg",
    ),
    School(
        school="michamuc",
        name="Michaeli-Gymnasium München",
        postcode="81671",
        city="München",
    ),
    School(
        school="mjsmuc",
        name="Staatliches Gymnasium Max-Josef-Stift München",
        postcode="81677",
        city="München",
    ),
    School(
        school="mmgyvil",
        name="Maximilian-von-Montgelas-Gymnasium",
        postcode="84137",
        city="Vilsbiburg",
    ),
    School(
        school="mogymuc",
        name="Staatliches Gymnasium München-Moosach",
        postcode="80993",
        city="München",
    ),
    School(
        school="moosgym",
        name="Karl-Ritter-von-Frisch-Gymnasium",
        postcode="85368",
        city="Moosburg",
    ),
    School(
        school="mpgymel",
        name="Martin-Pollich-Gymnasium Mellrichstadt",
        postcode="97638",
        city="Mellrichstadt",
    ),
    School(
        school="mswas",
        name="Mittelschule Wasserburg am Inn",
        postcode="83512",
        city="Wasserburg am Inn",
    ),
    School(
        school="mtgyau",
        name="Maria-Theresia-Gymnasium Augsburg",
        postcode="86150",
        city="Augsburg",
    ),
    School(
        school="mtgyerl",
        name="Städtisches Marie-Therese-Gymnasium Erlangen",
        postcode="91054",
        city="Erlangen",
    ),
    School(
        school="muebegy",
        name="Gymnasium Münchberg",
        postcode="95213",
        city="Münchberg",
    ),
    School(
        school="muenomuc",
        name="Gymnasium München-Nord",
        postcode="80937",
        city="München",
    ),
    School(
        school="mwgyal",
        name="Maria-Ward-Gymnasium Altötting",
        postcode="84503",
        city="Altötting",
    ),
    School(
        school="mwgyau",
        name="Maria-Ward-Gymnasium Augsburg",
        postcode="86152",
        city="Augsburg",
    ),
    School(
        school="mwgygz",
        name="Maria-Ward-Gymnasium Günzburg",
        postcode="89312",
        city="Günzburg",
    ),
    School(
        school="mwnuegy",
        name="Maria-Ward-Gymnasium Nürnberg",
        postcode="90489",
        city="Nürnberg",
    ),
    School(
        school="mwnuers",
        name="Maria-Ward-Realschule Nürnberg",
        postcode="90489",
        city="Nürnberg",
    ),
    School(
        school="mwrsn",
        name="Maria Ward Realschule Neuhaus",
        postcode="94152",
    ),
    School(
        school="nailagym",
        name="Hochfranken-Gymnasium Naila",
        postcode="95119",
        city="Naila",
    ),
    School(
        school="neugynue",
        name="Neues Gymnasium Nürnberg",
        postcode="90478",
        city="Nürnberg",
    ),
    School(
        school="neutragym",
        name="Gymnasium Neutraubling",
        postcode="93073",
        city="Neutraubling",
    ),
    School(
        school="nfgymuc",
        name="Gymnasium Neufreimann",
        postcode="80805",
        city="München",
    ),
    School(
        school="nkgwh",
        name="Nikolaus-Kopernikus-Gymnasium Weißenhorn",
        postcode="89264",
        city="Weißenhorn",
    ),
    School(
        school="nymph",
        name="Nymphenburger Schulen",
        postcode="80638",
        city="München",
    ),
    School(
        school="ohagym",
        name="Gymnasium Oberhaching",
        postcode="82041",
        city="Oberhaching",
    ),
    School(
        school="ohgyerl",
        name="Ohm-Gymnasium Erlangen",
        postcode="91052",
        city="Erlangen",
    ),
    School(
        school="olchigym",
        name="Gymnasium Olching",
        postcode="82140",
        city="Olching",
    ),
    School(
        school="omgyneu",
        name="Oskar-Maria-Graf-Gymnasium",
        postcode="85375",
    ),
    School(
        school="opgypul",
        name="Otfried-Preußler-Gymnasium Pullach",
        postcode="82049",
    ),
    School(
        school="ovmgymuc",
        name="Oskar-von-Miller-Gymnasium München",
        postcode="80803",
        city="München",
    ),
    School(
        school="ovtgym",
        name="Otto-von-Taube-Gymnasium Gauting",
        postcode="82131",
        city="Gauting",
    ),
    School(
        school="pafgym",
        name="Schyren-Gymnasium",
        postcode="85276",
        city="Pfaffenhofen an der Ilm",
    ),
    School(
        school="peugyau",
        name="Peutinger-Gymnasium Augsburg",
        postcode="86152",
        city="Augsburg",
    ),
    School(
        school="pfgyher",
        name="Paul-Pfinzing-Gymnasium",
        postcode="91217",
        city="Hersbruck",
    ),
    School(
        school="plagyans",
        name="Platen-Gymnasium Ansbach",
        postcode="91522",
        city="Ansbach",
    ),
    School(
        school="prmpul",
        name="Erzbischöfliches Pater-Rupert-Mayer-Gymnasium Pullach",
        postcode="82049",
    ),
    School(
        school="prmpulgs",
        name="Erzbischöfliche Pater-Rupert-Mayer-Grundschule Pullach",
        postcode="82049",
    ),
    School(
        school="psmsgraf",
        name="Propst-Seyberer-Mittelschule Grafenau",
    ),
    School(
        school="puheigy",
        name="Gymnasium Puchheim",
        postcode="82178",
        city="Puchheim",
    ),
    School(
        school="riegywue",
        name="Riemenschneider-Gymnasium Würzburg",
        postcode="97070",
        city="Würzburg",
    ),
    School(
        school="riwagybay",
        name="Richard-Wagner-Gymnasium Bayreuth",
        postcode="95444",
        city="Bayreuth",
    ),
    School(
        school="roewuegy",
        name="Röntgen-Gymnasium Würzburg",
        postcode="97070",
        city="Würzburg",
    ),
    School(
        school="rottlaugy",
        name="Rottmayr-Gymnasium Laufen",
        postcode="83410",
        city="Laufen",
    ),
    School(
        school="rsache",
        name="Achental-Realschule Marquartstein",
    ),
    School(
        school="rsgcham",
        name="Robert-Schuman-Gymnasium Cham",
    ),
    School(
        school="rsvat",
        name="Staatliche Realschule Vaterstetten",
        postcode="85598",
        city="Vaterstetten",
    ),
    School(
        school="rudigyau",
        name="Rudolf-Diesel-Gymnasium Augsburg",
        postcode="86163",
        city="Augsburg",
    ),
    School(
        school="rugymue",
        name="Ruperti-Gymnasium",
        postcode="84453",
        city="Mühldorf a. Inn",
    ),
    School(
        school="seligla",
        name="Gymnasium der Schulstiftung Seligenthal",
        postcode="84034",
        city="Landshut",
    ),
    School(
        school="seliglags",
        name="Grundschule der Schulstiftung Seligenthal",
        postcode="84034",
        city="Landshut",
    ),
    School(
        school="seligws",
        name="Wirtschaftsschule der Schulstiftung Seligenthal Landshut",
        postcode="84034",
        city="Landshut",
    ),
    School(
        school="shr",
        name="Samuel-Heinicke-Realschule",
        postcode="80992",
        city="München",
    ),
    School(
        school="siewuegym",
        name="Siebold-Gymnasium Würzburg",
        postcode="97070",
        city="Würzburg",
    ),
    School(
        school="simagygu",
        name="Simon-Marius-Gymnasium",
        postcode="91710",
        city="Gunzenhausen",
    ),
    School(
        school="sontgym",
        name="Gymnasium Sonthofen",
        postcode="87527",
        city="Sonthofen",
    ),
    School(
        school="sosgymuc",
        name="Städtisches Sophie-Scholl-Gymnasium München",
        postcode="80804",
        city="München",
    ),
    School(
        school="spmshau",
        name="Sport-Mittelschule Hauzenberg",
    ),
    School(
        school="stagymu",
        name="Staffelsee-Gymnasium Murnau",
        postcode="82418",
        city="Murnau am Staffelsee",
    ),
    School(
        school="stbgym",
        name="Gymnasium Starnberg",
        postcode="82319",
        city="Starnberg",
    ),
    School(
        school="stgogyna",
        name="St.-Gotthard-Gymnasium",
        postcode="94557",
    ),
    School(
        school="stugylen",
        name="Erzbischöfliches St.-Ursula-Gymnasium",
        postcode="50321",
        city="Brühl",
    ),
    School(
        school="tassimgy",
        name="Tassilo-Gymnasium Simbach am Inn",
        postcode="84359",
        city="Simbach",
    ),
    School(
        school="tgamuc",
        name="Theresia-Gerhardinger-Gymnasium am Anger",
        postcode="80331",
        city="München",
    ),
    School(
        school="tgrs",
        name="Erzbischöfliche Theresia-Gerhardinger-Mädchenrealschule",
        postcode="81541",
        city="München",
    ),
    School(
        school="theheugym",
        name="Theodor-Heuss-Gymnasium Nördlingen",
        postcode="86720",
        city="Nördlingen",
    ),
    School(
        school="theogymuc",
        name="Städtisches Theodolinden-Gymnasium München",
        postcode="81547",
        city="München",
    ),
    School(
        school="thergymuc",
        name="Theresien-Gymnasium München",
        postcode="80336",
        city="München",
    ),
    School(
        school="thomagymuc",
        name="Städtisches Thomas-Mann-Gymnasium München",
        postcode="81379",
        city="München",
    ),
    School(
        school="toelgvs",
        name="Gabriel-von-Seidl-Gymnasium Bad Tölz",
        postcode="83646",
        city="Bad Tölz",
    ),
    School(
        school="trugymuc",
        name="Gymnasium Trudering",
        postcode="81827",
        city="München",
    ),
    School(
        school="ufggym",
        name="Gymnasium Unterföhring",
        postcode="85774",
        city="Unterföhring",
    ),
    School(
        school="ugriegym",
        name="Gymnasium Untergriesbach",
        postcode="94107",
    ),
    School(
        school="urstragym",
        name="Gymnasium der Ursulinen-Schulstiftung Straubing",
        postcode="94315",
        city="Straubing",
    ),
    School(
        school="veihoegym",
        name="Gymnasium Veitshöchheim",
        postcode="97209",
    ),
    School(
        school="vhgybo",
        name="Veit-Höser-Gymnasium Bogen",
        postcode="94327",
        city="Bogen",
    ),
    School(
        school="visgyffb",
        name="Viscardi-Gymnasium Fürstenfeldbruck",
        postcode="82256",
        city="Fürstenfeldbruck",
    ),
    School(
        school="wbgl",
        name="Wolfgang-Borchert-Gymnasium Langenzenn",
        postcode="90579",
        city="Langenzenn",
    ),
    School(
        school="weilgym",
        name="Gymnasium Weilheim i. OB",
        postcode="82362",
        city="Weilheim in Oberbayern",
    ),
    School(
        school="welfen",
        name="Welfen-Gymnasium Schongau",
        postcode="86956",
        city="Schongau",
    ),
    School(
        school="wggselb",
        name="Walter-Gropius-Gymnasium Selb",
        postcode="95100",
        city="Selb",
    ),
    School(
        school="wggymuc",
        name="Städtisches Willi-Graf-Gymnasium München",
        postcode="80804",
        city="München",
    ),
    School(
        school="whgga",
        name="Werner-Heisenberg-Gymnasium Garching",
        postcode="85748",
        city="Garching bei München",
    ),
    School(
        school="whgmuc",
        name="Wilhelm-Hausenstein-Gymnasium München",
        postcode="81925",
        city="München",
    ),
    School(
        school="widipogy",
        name="Wilhelm-Diess-Gymnasium Pocking",
        postcode="94060",
        city="Pocking",
    ),
    School(
        school="wigywue",
        name="Wirsberg-Gymnasium Würzburg",
        postcode="97070",
        city="Würzburg",
    ),
    School(
        school="willigym",
        name="Willibald-Gluck-Gymnasium",
        postcode="92318",
        city="Neumarkt in der Oberpfalz",
    ),
    School(
        school="witgymuc",
        name="Wittelsbacher-Gymnasium München",
        postcode="80335",
        city="München",
    ),
    School(
        school="wkrstauf",
        name="Walter-Klingenbeck-Schule Taufkirchen",
        postcode="82024",
        city="Taufkirchen",
    ),
    School(
        school="wmrstrau",
        name="Walter-Mohr-Realschule Traunreut",
        postcode="83301",
        city="Traunreut",
    ),
    School(
        school="wvsgsau",
        name="Werner-von-Siemens-Grundschule Augsburg",
        postcode="86163",
        city="Augsburg",
    ),
    School(
        school="wvsgymuc",
        name="Städtisches Werner-von-Siemens-Gymnasium München",
        postcode="81735",
        city="München",
    ),
    School(
        school="wwgybay",
        name="Wirtschaftswissenschaftliches und Naturwissenschaftlich-technologisches Gymnasium der Stadt Bayreuth",
        postcode="95448",
        city="Bayreuth",
    ),
]



"""School module"""

import dataclasses

@dataclasses.dataclass
class School():
    """Dataclass representing a school"""
    school: str = None
    name: str = None
    postcode: str = None
    city: str = None



"""Elternprotal API"""

# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-arguments
# pylint: disable=too-many-positional-arguments
# pylint: disable=too-many-locals
# pylint: disable=too-many-statements
# pylint: disable=too-many-branches

from __future__ import annotations

import datetime
import logging
import re
import socket
from typing import Any, Dict
import urllib.parse

import aiohttp
import bs4
import pytz

from .const import (
    DEFAULT_REGISTER_SHOW_EMPTY,
    DEFAULT_REGISTER_START_MAX,
    DEFAULT_REGISTER_START_MIN,
    SCHOOL_SUBJECTS,
)

from .exception import (
    BadCredentialsException,
    CannotConnectException,
    StudentListException,
    ResolveHostnameException,
)

from .school import School
from .schools import SCHOOLS

from .appointment import Appointment
from .lesson import Lesson
from .letter import Letter
from .poll import Poll
from .student import Student
from .register import Register
from .sicknote import SickNote

_LOGGER = logging.getLogger(__name__)

type ConfigType = Dict[str, str]
type OptionType = Dict[str, Any]


class ElternPortalAPI:
    """API to retrieve the data."""

    def __init__(self):
        """Initialize the API."""

        self.timezone = pytz.timezone("Europe/Berlin")
        self.beautiful_soup_parser = "html5lib"

        # set_config
        self.school: str = None
        self.username: str = None
        self.password: str = None
        self.hostname: str = None
        self.base_url: str = None

        # set_option
        self.appointment: bool = False
        self.lesson: bool = False
        self.letter: bool = False
        self.poll: bool = False
        self.register: bool = False
        self.sicknote: bool = False

        # set_option_register
        self.register_start_min: int = DEFAULT_REGISTER_START_MIN
        self.register_start_max: int = DEFAULT_REGISTER_START_MAX
        self.register_show_empty: bool = DEFAULT_REGISTER_SHOW_EMPTY

        # async_validate_config
        self.ip: str = None
        self.session: aiohttp.ClientSession = None
        self.csrf: str = None
        self.school_name: str = None

        # other
        self.student: Student = None
        self.students: list[Student] = []
        self.last_update = None

    def set_config(self, school: str, username: str, password: str):
        """Initialize the config."""
        school = (
            school.lower()
            .strip()
            .removeprefix("https://")
            .removeprefix("http://")
            .removesuffix("/")
            .removesuffix(".eltern-portal.org")
        )

        if not re.match(r"^[A-Za-z0-9]{1,10}$", school):
            message = '"school" is wrong: one to ten alpha-numeric characters'
            raise BadCredentialsException(message)

        username = username.lower().strip()
        password = password.strip()
        hostname = school + ".eltern-portal.org"
        base_url = "https://" + hostname + "/"

        self.school = school
        self.username = username
        self.password = password
        self.hostname = hostname
        self.base_url = base_url

    def set_config_data(self, config: ConfigType) -> None:
        """Initialize the config data."""

        school = config.get("school")
        username = config.get("username")
        password = config.get("password")
        self.set_config(school, username, password)

    def set_option(
        self,
        appointment: bool = False,
        lesson: bool = False,
        letter: bool = False,
        poll: bool = False,
        register: bool = False,
        sicknote: bool = False,
    ) -> None:
        """Initialize the option."""

        self.appointment: bool = appointment
        self.lesson: bool = lesson
        self.letter: bool = letter
        self.poll: bool = poll
        self.register: bool = register
        self.sicknote: bool = sicknote

    def set_option_register(
        self,
        register_start_min: int = DEFAULT_REGISTER_START_MIN,
        register_start_max: int = DEFAULT_REGISTER_START_MAX,
        register_show_empty: int = DEFAULT_REGISTER_SHOW_EMPTY,
    ) -> None:
        """Initialize the option register."""

        self.register_start_min: int = register_start_min
        self.register_start_max: int = register_start_max
        self.register_show_empty: bool = register_show_empty

    def set_option_data(self, option: OptionType) -> None:
        """Initialize the option data."""

        appointment: bool = option.get("appointment", False)
        lesson: bool = option.get("lesson", False)
        letter: bool = option.get("letter", False)
        poll: bool = option.get("poll", False)
        register: bool = option.get("register", False)
        sicknote: bool = option.get("sicknote", False)

        register_start_min: int = option.get(
            "register_start_min", DEFAULT_REGISTER_START_MIN
        )
        register_start_max: int = option.get(
            "register_start_max", DEFAULT_REGISTER_START_MAX
        )
        register_show_empty: int = option.get(
            "register_show_empty", DEFAULT_REGISTER_SHOW_EMPTY
        )

        self.set_option(appointment, lesson, letter, poll, register, sicknote)
        self.set_option_register(
            register_start_min, register_start_max, register_show_empty
        )

    async def async_validate_config(self) -> None:
        """Function validate configuration."""
        if self.school == "demo":
            # base
            self.ip = "127.0.0.1"
            self.csrf = "demo"
            self.school_name = "Demo Gymnasium"

            # login
            student_id = "demo"
            fullname = "Erika Mustermann (1a)"
            self.students = [Student(student_id, fullname)]
            return

        _LOGGER.debug("Try to resolve hostname %s", self.hostname)
        try:
            self.ip = socket.gethostbyname(self.hostname)
        except socket.gaierror as sge:
            message = f"Cannot resolve hostname {self.hostname}"
            _LOGGER.exception(message)
            raise ResolveHostnameException(message) from sge
        _LOGGER.debug("IP address is %s", self.ip)

        async with aiohttp.ClientSession(self.base_url) as self.session:
            await self.async_base()
            await self.async_login()
            await self.async_logout()

    async def async_update(self) -> None:
        """Elternportal start page."""
        if self.school == "demo":

            student_id = "demo"
            fullname = "Erika Mustermann (1a)"
            self.student = Student(student_id, fullname)
            self.students = [self.student]

            day0 = datetime.datetime.now().date()
            day0 -= datetime.timedelta(days=day0.weekday())
            day1 = day0 + datetime.timedelta(days=1)
            day2 = day0 + datetime.timedelta(days=2)
            day3 = day0 + datetime.timedelta(days=3)
            day4 = day0 + datetime.timedelta(days=4)
            day7 = day0 + datetime.timedelta(days=7)

            # appointment
            self.student.appointments = [
                Appointment(
                    appointment_id="id_1",
                    title="Schulaufgabe in Englisch",
                    short="SA in E",
                    classname="event-important",
                    start=day3,
                    end=day3,
                ),
                Appointment(
                    appointment_id="id_2",
                    title="Schulaufgabe in Deutsch",
                    short="SA in D",
                    classname="event-important",
                    start=day7,
                    end=day7,
                ),
            ]

            # lesson
            self.student.lessons = []

            # letter
            self.student.letters = [
                Letter(
                    letter_id="1",
                    number="#1",
                    sent=datetime.datetime.now(),
                    new=True,
                    attachment=True,
                    subject="1. Klassenelternabend",
                    distribution="1a",
                    description="Liebe Eltern\n\n...\n",
                )
            ]

            # poll
            self.student.polls = []

            # register
            self.student.registers = [
                Register(
                    subject="Englisch",
                    short="E",
                    teacher="Herr Mustermann",
                    lesson="single",
                    substitution=False,
                    empty=False,
                    rtype="homework",
                    start=day0,
                    completion=day2,
                    description="Buch S. 123, Aufgaben 1-3",
                ),
                Register(
                    subject="Deutsch",
                    short="D",
                    teacher="Frau Müller",
                    lesson="double",
                    substitution=True,
                    empty=False,
                    rtype="homework",
                    start=day1,
                    completion=day4,
                    description="Arbeitsheft S. 45, Aufgaben 1 b+c",
                ),
            ]

            # sicknote
            self.student.sicknotes = [SickNote(day1, day1, None)]

            self.last_update = datetime.datetime.now()
            return

        async with aiohttp.ClientSession(self.base_url) as self.session:

            await self.async_base()
            await self.async_login()

            for self.student in self.students:
                await self.async_set_child()

                if self.appointment:
                    await self.async_appointment()

                if self.lesson:
                    await self.async_lesson()

                if self.letter:
                    await self.async_letter()

                if self.poll:
                    await self.async_poll()

                if self.register:
                    await self.async_register()

                if self.sicknote:
                    await self.async_sicknote()

            await self.async_logout()
            self.last_update = datetime.datetime.now()

    async def async_base(self) -> None:
        """Elternportal base."""

        url = "/"
        _LOGGER.debug("base.url=%s", url)
        async with self.session.get(url) as response:
            if response.status != 200:
                message = f"base.status={response.status}"
                _LOGGER.exception(message)
                raise CannotConnectException(message)

            html = await response.text()
            if "Dieses Eltern-Portal existiert nicht" in html:
                message = f"The elternportal {self.base_url} does not exist."
                _LOGGER.exception(message)
                raise CannotConnectException(message)

            soup = bs4.BeautifulSoup(html, self.beautiful_soup_parser)

            try:
                tag = soup.find("input", {"name": "csrf"})
                csrf = tag["value"]
                self.csrf = csrf
            except TypeError as te:
                message = "The 'input' tag with the name 'csrf' could not be found."
                _LOGGER.exception(message)
                raise CannotConnectException(message) from te

            try:
                tag = soup.find("h2", {"id": "schule"})
                school_name = tag.get_text()
                self.school_name = school_name
            except TypeError as te:
                message = "The 'h2' tag with the id 'schule' could not be found."
                _LOGGER.exception(message)
                raise CannotConnectException(message) from te

    async def async_login(self) -> None:
        """Elternportal login."""

        url = "/includes/project/auth/login.php"
        _LOGGER.debug("login.url=%s", url)
        login_data = {
            "csrf": self.csrf,
            "username": self.username,
            "password": self.password,
            "go_to": "",
        }
        async with self.session.post(url, data=login_data) as response:
            if response.status != 200:
                message = f"login.status={response.status}"
                _LOGGER.exception(message)
                raise CannotConnectException(message)

            html = await response.text()
            soup = bs4.BeautifulSoup(html, self.beautiful_soup_parser)

            tag = soup.select_one(".pupil-selector")
            if tag is None:
                message = "The tag with class 'pupil-selector' could not be found."
                raise BadCredentialsException(message)

            self.students = []
            tags = soup.select(".pupil-selector select option")
            if not tags:
                message = "The select options could not be found."
                raise StudentListException(message)

            for tag in tags:
                try:
                    student_id = tag["value"]
                except Exception as e:
                    message = (
                        "The 'value' atrribute of a pupil option could not be found."
                    )
                    raise StudentListException() from e

                try:
                    fullname = tag.get_text().strip()
                except Exception as e:
                    message = "The 'text' of a pupil option could not be found."
                    raise StudentListException() from e

                self.student = Student(student_id, fullname)
                self.students.append(self.student)

    async def async_set_child(self) -> None:
        """Elternportal set child."""

        url = "/api/set_child.php?id=" + self.student.student_id
        _LOGGER.debug("set_child.url=%s", url)
        async with self.session.post(url) as response:
            if response.status != 200:
                _LOGGER.debug("set_child.status=%s", response.status)

    async def async_appointment(self) -> None:
        """Elternportal appointment."""

        self.student.appointments = []
        url = "/api/ws_get_termine.php"
        _LOGGER.debug("appointment.url=%s", url)
        async with self.session.get(url) as response:
            if response.status != 200:
                _LOGGER.debug("appointment.status=%s", response.status)

            # process malformed JSON response with parameter content_type
            json = await response.json(content_type="text/html")
            for result in json["result"]:
                start = int(str(result["start"])[0:-3])
                start = datetime.datetime.fromtimestamp(start, self.timezone).date()
                end = int(str(result["end"])[0:-3])
                end = datetime.datetime.fromtimestamp(end, self.timezone).date()

                appointment = Appointment(
                    result["id"],
                    result["title"],
                    result["title_short"],
                    result["class"],
                    start,
                    end,
                )
                self.student.appointments.append(appointment)

    async def async_lesson(self) -> None:
        """Elternportal lesson."""

        self.student.lessons = []
        url = "/service/stundenplan"
        _LOGGER.debug("lesson.url=%s", url)
        async with self.session.get(url) as response:
            if response.status != 200:
                _LOGGER.debug("lesson.status=%s", response.status)
            html = await response.text()
            soup = bs4.BeautifulSoup(html, self.beautiful_soup_parser)

            table_rows = soup.select("#asam_content div.table-responsive table tr")
            for table_row in table_rows:
                table_cells = table_row.select("td")

                if len(table_cells) == 6:
                    # Column 0
                    lines = table_cells[0].find_all(string=True)
                    number = lines[0] if len(lines) > 0 else ""
                    # time = lines[1] if len(lines) > 1 else ""

                    # Column 1-5: Monday to Friday
                    for weekday in range(1, 5):
                        span = table_cells[weekday].select_one("span span")
                        if span:
                            lines = span.find_all(string=True)
                            subject = lines[0].strip() if len(lines) > 0 else ""
                            room = lines[1].strip() if len(lines) > 1 else ""

                            if subject != "":
                                lesson = Lesson(weekday, number, subject, room)
                                self.student.lessons.append(lesson)

    async def async_letter(self) -> None:
        """Elternportal letter."""

        self.student.letters = []
        url = "/aktuelles/elternbriefe"
        _LOGGER.debug("letter.url=%s", url)
        async with self.session.get(url) as response:
            if response.status != 200:
                _LOGGER.debug("letter.status=%s", response.status)
            html = await response.text()
            soup = bs4.BeautifulSoup(html, self.beautiful_soup_parser)

            tags = soup.select(".link_nachrichten")
            for tag in tags:
                # letter id
                match = re.search(r"\d+", tag.get("onclick"))
                letter_id = match[0] if match else None

                # attachment
                attachment = tag.name == "a"

                # sent
                match = re.search(
                    r"\d{2}\.\d{2}\.\d{4} \d{2}:\d{2}:\d{2}", tag.get_text()
                )
                if match is None:
                    sent = None
                else:
                    try:
                        sent = datetime.datetime.strptime(match[0], "%d.%m.%Y %H:%M:%S")
                        sent = self.timezone.localize(sent)
                    except ValueError:
                        sent = None

                # new + number
                cell = soup.find("td", {"id": "empf_" + letter_id})
                if cell is None:
                    new = True
                    number = "???"
                else:
                    new = cell.get_text() == "Empfang noch nicht bestätigt."
                    cell2 = cell.find_previous_sibling()
                    if cell2 is None:
                        number = "???"
                    else:
                        number = cell2.get_text()

                # subject
                cell = tag.find("h4")
                subject = cell.get_text() if cell else None

                # distribution + description
                cell = tag.parent
                if cell is None:
                    distribution = None
                    description = None
                else:
                    span = cell.select_one("span[style='font-size: 8pt;']")
                    if span is None:
                        distribution = None
                    else:
                        text = span.get_text()
                        liste = text.split("Klasse/n: ")
                        liste = [x for x in liste if x]
                        distribution = ", ".join(liste)

                    lines = cell.find_all(string=True)
                    description = ""
                    skip = True
                    for i in range(1, len(lines)):
                        line = lines[i].replace("\r", "").replace("\n", "")
                        if not skip:
                            description += line + "\n"
                        if line.startswith("Klasse/n: "):
                            skip = False

                letter = Letter(
                    letter_id=letter_id,
                    number=number,
                    sent=sent,
                    new=new,
                    attachment=attachment,
                    subject=subject,
                    distribution=distribution,
                    description=description,
                )
                self.student.letters.append(letter)

    async def async_poll(self) -> None:
        """Elternportal poll."""

        self.student.polls = []
        url = "/aktuelles/umfragen"
        _LOGGER.debug("poll.url=%s", url)
        async with self.session.get(url) as response:
            if response.status != 200:
                _LOGGER.debug("poll.status=%s", response.status)
            html = await response.text()
            soup = bs4.BeautifulSoup(html, self.beautiful_soup_parser)

            base_tag = soup.find("base")
            baseurl = base_tag["href"] if base_tag else url

            rows = soup.select("#asam_content div.row.m_bot")
            for row in rows:
                tag = row.select_one("div div:nth-child(1) a.umf_list")
                if tag is None:
                    title = None
                    href = None
                else:
                    title = tag.get_text()
                    href = urllib.parse.urljoin(baseurl, tag["href"])

                tag = row.select_one("div div:nth-child(1) a[title='Anhang']")
                attachment = tag is not None

                tag = row.select_one("div div:nth-child(2)")
                if tag is None:
                    end = None
                else:
                    match = re.search(r"\d{2}\.\d{2}\.\d{4}", tag.get_text())
                    if match is None:
                        end = None
                    else:
                        end = datetime.datetime.strptime(match[0], "%d.%m.%Y").date()

                tag = row.select_one("div div:nth-child(3)")
                if tag is None:
                    vote = None
                else:
                    match = re.search(r"\d{2}\.\d{2}\.\d{4}", tag.get_text())
                    if match is None:
                        vote = None
                    else:
                        vote = datetime.datetime.strptime(match[0], "%d.%m.%Y").date()

                if href is None:
                    detail = None
                else:
                    async with self.session.get(href) as response2:
                        html2 = await response2.text()
                        soup2 = bs4.BeautifulSoup(html2, self.beautiful_soup_parser)

                        div2 = soup2.select_one(
                            "#asam_content form.form-horizontal div.form-group:nth-child(3)"
                        )
                        detail = div2.get_text() if div2 else None

                poll = Poll(
                    title=title,
                    href=href,
                    attachment=attachment,
                    vote=vote,
                    end=end,
                    detail=detail,
                )
                self.student.polls.append(poll)

    async def async_register(self) -> None:
        """Elternportal register."""

        self.student.registers = []
        date_current = datetime.date.today() + datetime.timedelta(
            days=self.register_start_min
        )
        date_until = datetime.date.today() + datetime.timedelta(
            days=self.register_start_max
        )
        while date_current <= date_until:

            url = "/service/klassenbuch?cur_date=" + date_current.strftime("%d.%m.%Y")
            _LOGGER.debug("register.url=%s", url)
            async with self.session.get(url) as response:
                if response.status != 200:
                    _LOGGER.debug("register.status=%s", response.status)
                html = await response.text()
                soup = bs4.BeautifulSoup(html, self.beautiful_soup_parser)

                tags = soup.select("#asam_content table.table.table-bordered")
                for tag in tags:
                    table_cells = tag.select("th")
                    content = table_cells[1].get_text() if len(table_cells) > 1 else ""
                    subject = None
                    short = None
                    teacher = None
                    lesson = None
                    substitution = False
                    match = re.search(
                        r"(.*) - Lehrkraft: (.*) \((Einzel|Doppel)stunde(, Vertretung)?\)",
                        content,
                    )
                    if match:
                        subject = match[1].replace("Fach: ", "")
                        teacher = match[2]
                        lesson = (
                            match[3]
                            .replace("Einzel", "single")
                            .replace("Doppel", "double")
                        )
                        substitution = match[4] is not None

                    for school_subject in SCHOOL_SUBJECTS:
                        if school_subject["Name"] == subject:
                            short = school_subject["Short"]

                    table_cells = tag.select("td")
                    rtype = table_cells[0].get_text() if len(table_cells) > 0 else ""
                    rtype = rtype.replace("Hausaufgabe", "homework")

                    lines = table_cells[1].find_all(string=True)
                    description = lines[0] if len(lines) > 0 else None

                    empty = (
                        not description
                        or description == "Keine Hausaufgabe eingetragen."
                    )

                    date_completion = date_current
                    if len(lines) > 2:
                        match = re.search(r"\d{2}\.\d{2}\.\d{4}", lines[2])
                        if match:
                            date_completion = datetime.datetime.strptime(
                                match[0], "%d.%m.%Y"
                            ).date()

                    if self.register_show_empty or not empty:
                        register = Register(
                            subject=subject,
                            short=short,
                            teacher=teacher,
                            lesson=lesson,
                            substitution=substitution,
                            empty=empty,
                            rtype=rtype,
                            start=date_current,
                            completion=date_completion,
                            description=description,
                        )
                        self.student.registers.append(register)

            date_current += datetime.timedelta(days=1)

    async def async_sicknote(self) -> None:
        """Elternportal sick note."""

        self.student.sicknotes = []
        url = "/meldungen/krankmeldung"
        _LOGGER.debug("sicknote.url=%s", url)
        async with self.session.get(url) as response:
            if response.status != 200:
                _LOGGER.debug("sicknote.status=%s", response.status)
            html = await response.text()

            soup = bs4.BeautifulSoup(html, self.beautiful_soup_parser)

            rows = soup.select("#asam_content table.ui.table tr")
            for row in rows:
                cells = row.select("td")

                # link
                try:
                    tag = cells[0].find("a")
                    link = tag["href"]
                except TypeError:
                    link = None

                # query
                result = urllib.parse.urlparse(link)
                query = urllib.parse.parse_qs(result.query)

                # df -> start
                start = None
                if "df" in query:
                    df = int(query["df"][0])
                    start = datetime.datetime.fromtimestamp(df, self.timezone).date()
                else:
                    if len(cells) > 1:
                        lines = cells[1].find_all(string=True)
                        if lines:
                            match = re.search(r"\d{2}\.\d{2}\.\d{4}", lines[0])
                            if match:
                                start = datetime.datetime.strptime(
                                    match[0], "%d.%m.%Y"
                                ).date()

                # dt -> end
                end = start
                if "dt" in query:
                    dt = int(query["dt"][0])
                    end = datetime.datetime.fromtimestamp(dt, self.timezone).date()

                # k -> comment
                comment = None
                if "k" in query:
                    comment = str(query["k"][0])
                else:
                    if len(cells) > 2:
                        comment = cells[2].get_text()

                if comment == "":
                    comment = None
                sicknote = SickNote(start, end, comment)
                self.student.sicknotes.append(sicknote)

    async def async_logout(self) -> None:
        """Elternportal logout."""

        url = "/logout"
        _LOGGER.debug("logout.url=%s", url)
        async with self.session.get(url) as response:
            if response.status != 200:
                message = f"logout.status={response.status}"
                _LOGGER.exception(message)
                raise CannotConnectException(message)

    def get_schools(self) -> list[School]:
        """Elternportal get list of schools."""
        return SCHOOLS



""" Appointment module """

import dataclasses
import datetime

@dataclasses.dataclass
class Appointment():
    """Class representing an appointment"""
    appointment_id: str
    title: str
    short: str
    classname: str
    start: datetime.date
    end: datetime.date



""" Tool to update the school database."""

import datetime
import os
import pathlib
import json
import socket
import sys
import time

import bs4
import geopy
import requests

SLEEP_TIME = 5 # seconds

directory = pathlib.Path(__file__).parent.resolve()
parent = pathlib.Path(__file__).parent.parent.resolve()
path_html: str = os.path.join(directory, "schools.html")
path_json: str = os.path.join(directory, "schools.json")
path_md: str = os.path.join(directory, "SCHOOLS.md")
path_py: str = os.path.join(parent, "src", "pyelternportal", "schools.py")

year_month = datetime.date.today().isoformat()[:8]
geolocator = geopy.geocoders.Nominatim(user_agent="pyelternportal")

with open(path_json, mode="r", encoding="utf-8") as read_file:
    data = json.load(read_file)

for school in sys.argv[1:]:
    if next((s for s in data if s["school"] == school), None) is None:
        data.append({"school": school})

data.sort(key=lambda s: s["school"])

for s in data:
    if "domain" not in s and s["school"] is not None:
        s["domain"] = f"{s["school"]}.eltern-portal.org"

    if "url" not in s and s["domain"] is not None:
        s["url"] = f"https://{s["domain"]}"

    ipcheck = (
        "ip" not in s or "ip_lastcheck" not in s or year_month not in s["ip_lastcheck"]
    )
    if ipcheck and s["domain"] is not None:
        hostname = s["domain"]
        ip = socket.gethostbyname(hostname)
        print(f"resolve {hostname} to {ip}")
        s["ip"] = ip
        s["ip_lastcheck"] = datetime.date.today().isoformat()

    namecheck = (
        "name" not in s
        or "exists" not in s
        or "name_lastcheck" not in s
        or year_month not in s["name_lastcheck"]
    )
    if namecheck and s["url"] is not None:
        response = requests.get(url=s["url"], timeout=30)
        html = response.text
        if "Dieses Eltern-Portal existiert nicht" in html:
            print(f"resolve {s["url"]} to 'Eltern-Portal existiert nicht'")
            s["exists"] = False
            s["name"] = None
        else:
            soup = bs4.BeautifulSoup(html, "html5lib")
            tag = soup.find("h2", {"id": "schule"})
            name = tag.get_text() if tag is not None else None
            print(f"resolve {s["url"]} to {name}")
            s["exists"] = True
            s["name"] = name
        s["name_lastcheck"] = datetime.date.today().isoformat()
        time.sleep(SLEEP_TIME)

    geocheck = (
        "geo" not in s
        or "geo_lastcheck" not in s
        or year_month not in s["geo_lastcheck"]
    )
    if geocheck and s["name"] is not None:
        name_geo = s["name_geo"] if "name_geo" in s else s["name"]
        location = geolocator.geocode(
            name_geo, addressdetails=True, language="de", country_codes="de"
        )
        if location is None:
            print(f"resolve {name_geo} to None")
            s["geo"] = None
        else:
            geotype = location.raw["type"]
            address = location.raw["address"]
            if geotype == "school":
                postcode = address["postcode"]
                city =  address["city"] if "city" in address else address.get("town")
                print(f"resolve {name_geo} to {postcode} {city}")
                s["geo"] = {
                    "type": geotype,
                    "lat": location.raw["lat"],
                    "lon": location.raw["lon"],
                    "name": location.raw["name"],
                    "road": address["road"],
                    "house_number": address.get("house_number"),
                    "postcode": postcode,
                    "city": city,
                    "country": address["country"],
                }
            else:
                print(f"resolve {name_geo} to {geotype}")
                s["geo"] = {
                    "type": geotype,
                }
        s["geo_lastcheck"] = datetime.date.today().isoformat()
        time.sleep(SLEEP_TIME)


with open(path_md, mode="w", encoding="utf-8") as fh:
    fh.write("# Known instances of Eltern-Portal\n")
    fh.write("\n")
    fh.write("Identifier | Url                                   | School\n")
    fh.write(":--------- | :------------------------------------ | :-----\n")
    for s in data:
        if s.get("exists", False):
            identifier = s["school"]
            url = s.get("url")
            school = s.get("name")
            if "geo" in s and s["geo"] is not None:
                geo = s["geo"]
                if "postcode" in geo and geo["postcode"] is not None:
                    school += ", " + geo["postcode"]
                    if "city" in geo and geo["city"] is not None:
                        school += " " + geo["city"]
                else:
                    if "city" in geo and geo["city"] is not None:
                        school += ", " + geo["city"]

            fh.write(f"{identifier:<10} | {url:<37} | {school}\n")

with open(file=path_py, mode="w", encoding="utf-8") as fh:
    fh.write('""" Known instances of eltern-portal.org """\n\n')
    fh.write("# pylint: disable=too-many-lines\n\n")
    fh.write("from .school import School\n\n")
    fh.write("SCHOOLS = [\n")
    for s in data:
        if s.get("exists", False):
            fh.write("    School(\n")
            fh.write('        school="' + s["school"] + '",\n')
            if "name" in s and s["name"]:
                fh.write('        name="' + s["name"] + '",\n')
            if "geo" in s and s["geo"] and s["geo"]["type"] == "school":
                if s["geo"]["postcode"]:
                    fh.write('        postcode="' + s["geo"]["postcode"] + '",\n')
                if s["geo"]["city"]:
                    fh.write('        city="' + s["geo"]["city"] + '",\n')
            fh.write("    ),\n")
    fh.write("]\n")

with open(file=path_json, mode="w", encoding="utf-8") as fh:
    json.dump(obj=data, fp=fh, indent=2, sort_keys=True)


with open(file=path_html, mode="w", encoding="utf-8") as fh:
    fh.writelines('<!DOCTYPE html>')
    fh.writelines('<html lang="en">')
    fh.writelines('<head>')
    fh.writelines('    <base target="_top">')
    fh.writelines('    <meta charset="utf-8">')
    fh.writelines('    <meta name="viewport" content="width=device-width, initial-scale=1">')
    fh.writelines('    <title>Schools</title>')
    fh.writelines('    <link rel="stylesheet" href="https://unpkg.com/leaflet@1.9.4/dist/leaflet.css" integrity="sha256-p4NxAoJBhIIN+hmNHrzRCf9tD/miZyoHS5obTRR9BMY=" crossorigin="" />')
    fh.writelines('    <script src="https://unpkg.com/leaflet@1.9.4/dist/leaflet.js" integrity="sha256-20nQCchB9co0qIjJZRGuk2/Z9VM+kNiyxNV1lvTlZBo=" crossorigin=""></script>')
    fh.writelines('    <style>')
    fh.writelines('        html, body {')
    fh.writelines('            height: 100%;')
    fh.writelines('            margin: 0;')
    fh.writelines('        }')
    fh.writelines('    </style>')
    fh.writelines('</head>')
    fh.writelines('<body>')
    fh.writelines('    <div id="map" style="width: 100%; height: 100%;"></div>')
    fh.writelines('    <script>')
    fh.writelines("        const map = L.map('map').setView([51.163361, 10.447683], 7);")
    fh.writelines("        const tiles = L.tileLayer('https://tile.openstreetmap.org/{z}/{x}/{y}.png', {")
    fh.writelines("            maxZoom: 19,")
    fh.writelines("            attribution: '&copy; <a href=\"http://www.openstreetmap.org/copyright\">OpenStreetMap</a>'")
    fh.writelines("        }).addTo(map);")
    for s in data:
        if "geo" in s and s["geo"]:
            geo = s["geo"]
            if "lat" in geo and geo["lat"] and "lon" in geo and geo["lon"]:
                fh.writelines(f"        const m_{s["school"]} = L.marker([{geo["lat"]}, {geo["lon"]}]).bindTooltip('{s["school"]}: {s["name"]}').addTo(map);")
    fh.writelines('    </script>')
    fh.writelines('</body>')
    fh.writelines('</html>')



"""Poll module"""

import dataclasses
import datetime

@dataclasses.dataclass
class Poll:
    """Class representing a poll"""
    title: str
    href: str
    attachment: bool
    vote: datetime.datetime
    end: datetime.datetime
    detail: str



""" Register module """

# pylint: disable=too-many-instance-attributes

import dataclasses
import datetime

@dataclasses.dataclass
class Register():
    """Class representing a register"""
    subject: str
    short: str
    teacher: str
    lesson: str
    substitution: bool
    empty: bool
    rtype: str
    start: datetime.date
    completion: datetime.date
    description: str



""" Letter module """

# pylint: disable=too-many-instance-attributes

import dataclasses
import datetime

@dataclasses.dataclass
class Letter():
    """Class representing a letter"""
    letter_id: str
    number: str
    sent: datetime.datetime
    new: bool
    attachment: bool
    subject: str
    distribution: str
    description: str



"""Elternportal API - constants"""

from typing import Dict

DEFAULT_REGISTER_SHOW_EMPTY: bool = False
DEFAULT_REGISTER_START_MIN: int = -6
DEFAULT_REGISTER_START_MAX: int = +5

SCHOOL_SUBJECTS: list[Dict[str, str]] = [
    {"Short": "B", "Name": "Biologie"},
    {"Short": "BcP", "Name": "Biologisch-chemisches Praktikum"},
    {"Short": "C", "Name": "Chemie"},
    {"Short": "D", "Name": "Deutsch"},
    {"Short": "E", "Name": "Englisch"},
    {"Short": "Eth", "Name": "Ethik"},
    {"Short": "Ev", "Name": "Evangelische Religionslehre"},
    {"Short": "F", "Name": "Französisch"},
    {"Short": "G", "Name": "Geschichte"},
    {"Short": "Geo", "Name": "Geographie"},
    {"Short": "Geol", "Name": "Geologie"},
    {"Short": "Gr", "Name": "Griechisch"},
    {"Short": "Inf", "Name": "Informatik"},
    {"Short": "Infang", "Name": "Angewandte Informatik"},
    {"Short": "It", "Name": "Italienisch"},
    {"Short": "K", "Name": "Katholische Religionslehre"},
    {"Short": "Ku", "Name": "Kunst"},
    {"Short": "L", "Name": "Latein"},
    {"Short": "M", "Name": "Mathematik"},
    {"Short": "Mu", "Name": "Musik"},
    {"Short": "NT", "Name": "Natur und Technik"},
    {"Short": "Ph", "Name": "Physik"},
    {"Short": "PhAst", "Name": "Astrophysik"},
    {"Short": "Ru", "Name": "Russisch"},
    {"Short": "S", "Name": "Sport"},
    {"Short": "STheo", "Name": "Sporttheorie"},
    {"Short": "SpG", "Name": "Sozialpraktische Grundbildung"},
    {"Short": "Sk", "Name": "Sozialkunde"},
    {"Short": "Sp", "Name": "Spanisch"},
    {"Short": "SwA", "Name": "Sozialwissenschaftliche Arbeitsfelder"},
    {"Short": "WIn", "Name": "Wirtschaftsinformatik"},
    {"Short": "WR", "Name": "Wirtschaft und Recht"},
]



""" Elternportal API - exceptions """

class BadCredentialsException(Exception):
    """Error to indicate there are bad credentials."""


class CannotConnectException(Exception):
    """Error to indicate we cannot connect."""


class StudentListException(Exception):
    """Error to indicate there are no students."""


class ResolveHostnameException(Exception):
    """Error to indicate we cannot resolve the hostname."""



"""Student module"""

# pylint: disable=too-many-instance-attributes

import re

from .appointment import Appointment
from .lesson import Lesson
from .letter import Letter
from .poll import Poll
from .register import Register
from .sicknote import SickNote

class Student():
    """Class representing a student"""
    def __init__(self, student_id: str, fullname: str):

        try:
            match = re.search(r"^(\S+)\s+(.*)\s+\((\S+)\)$", fullname)
            firstname = match[1]
            lastname = match[2]
            classname = match[3]
        except TypeError:
            firstname = f"S{student_id}"
            lastname = None
            classname = None

        self.student_id: str = student_id
        self.fullname: str = fullname
        self.firstname: str = firstname
        self.lastname: str = lastname
        self.classname: str = classname

        self.appointments: list = []
        self.lessons: list = []
        self.letters: list = []
        self.polls: list = []
        self.registers: list = []
        self.sicknotes: list = []

    def get_id(self) -> str:
        """Get student id"""
        return self.student_id

    def get_fullname(self) -> str:
        """Get full name"""
        return self.fullname

    def get_firstname(self) -> str:
        """Get first name"""
        return self.firstname

    def get_count(self) -> int:
        """Get count of all sequences"""
        return (
            len(self.appointments)
            + len(self.lessons)
            + len(self.letters)
            + len(self.polls)
            + len(self.registers)
            + len(self.sicknotes)
        )

    def set_appointments(self, appointments: list[Appointment]) -> None:
        """set list of appointments"""
        self.appointments = appointments

    def get_appointments(self) -> list[Appointment]:
        """get list of appointments"""
        return self.appointments

    def set_lessons(self, lessons: list[Lesson]) -> None:
        """set list of lessons"""
        self.lessons = lessons

    def get_lessons(self) -> list[Lesson]:
        """get list of lessons"""
        return self.lessons

    def set_letters(self, letters: list[Letter]) -> None:
        """set list of letters"""
        self.letters = letters

    def get_letters(self) -> list[Letter]:
        """get list of letters"""
        return self.letters

    def set_polls(self, polls: list[Poll]) -> None:
        """set list of polls"""
        self.polls = polls

    def get_polls(self) -> list[Poll]:
        """get list of polls"""
        return self.polls

    def set_registers(self, registers: list[Register]) -> None:
        """set list of registers"""
        self.registers = registers

    def get_registers(self) -> list[Register]:
        """get list of registers"""
        return self.registers

    def set_sicknotes(self, sicknotes: list[SickNote]) -> None:
        """set list of sicknotes"""
        self.sicknotes = sicknotes

    def get_sicknotes(self) -> list[SickNote]:
        """get list of sicknotes"""
        return self.sicknotes



""" Sick note module """

import dataclasses
import datetime

@dataclasses.dataclass
class SickNote():
    """Class representing a sick note"""
    start: datetime.date
    end: datetime.date
    comment: str
